/* com.cintegrity.Lib.Collection.TestObjectObjectListMap.p */

define temp-table tt_Trx no-undo
                                                            /* unique values */
    field in_Seq as integer
    field ch_CustID as character
    /* omitted unique decimal */
    /* omitted unique date */
    field dt_TrxTimeStamp as datetime
                                                        /* non-unique values */
    field in_Quantity as integer
    field ch_CustName as character
    field ch_ItemName as character
    field de_TrxAmount as decimal
    field da_TrxDate as date
    field dt_PostTime as datetime
    index ix_BySeq as primary unique in_Seq
    index ix_ByTimeStamp as unique dt_TrxTimeStamp
    .
define temp-table tt_Cust no-undo
    field ch_CustName as character
    field ob_CustID as Progress.Lang.Object
    index by_Name as primary unique ch_CustName.
define variable lhn_Trx as handle no-undo.
define variable lob_MyList as com.cintegrity.Lib.Collection.ObjectObjectListMap no-undo.
define variable lob_CurrentTrx as com.cintegrity.Lib.Collection.Trx no-undo.
define variable lob_OldTrx as Progress.Lang.Object.
define variable lob_FirstTrx as class com.cintegrity.Lib.Collection.Trx no-undo.
define variable lob_LastTrx as class com.cintegrity.Lib.Collection.Trx no-undo.
define variable lch_InputFileName as character no-undo initial "com/cintegrity/Lib/Collection/TrxData.xml".
define variable lch_SchemaFileName as character no-undo initial "com/cintegrity/Lib/Collection/TrxDataSchema.xml".
define variable lob_CurrentKey as class com.cintegrity.Lib.Collection.CustID no-undo.
define variable lob_Cust as class com.cintegrity.Lib.Collection.CustID no-undo.
define variable lin_TargetCount as integer no-undo.
define variable lin_Count as integer no-undo.
define variable llg_IsOK as logical no-undo.
define variable lin_CurrentSeq as integer no-undo.

if search( lch_InputFileName ) ne ?
then lch_InputFileName = search( lch_InputFileName ).
else do:
  message "Could not find test input file " + lch_InputFileName + " - aborting" view-as alert-box.
  return.
end.

if search( lch_SchemaFileName ) ne ?
then lch_SchemaFileName = search( lch_SchemaFileName ).
else do:
  message "Could not find Schema file " + lch_SchemaFileName + " - aborting" view-as alert-box.
  return.
end.

lhn_Trx = temp-table tt_Trx:handle.
lhn_Trx:read-xml(
    "FILE",
    lch_InputFileName,
    "empty",
    lch_SchemaFileName,
    false
    ).
for each tt_Trx:
  find tt_Cust
      where tt_Cust.ch_CustName EQ tt_Trx.ch_CustName
      no-error.
  if not available( tt_Cust )
  then do:
    create tt_Cust.
    tt_Cust.ch_CustName = tt_Trx.ch_CustName.
    tt_Cust.ob_Cust = new com.cintegrity.Lib.Collection.CustID(
       tt_Trx.ch_CustName
       ).
  end.
end.

lob_MyList = new com.cintegrity.Lib.Collection.ObjectObjectListMap().

for each tt_Trx:
  lob_CurrentTrx = new com.cintegrity.Lib.Collection.Trx(
      input tt_Trx.in_Seq,
      input tt_Trx.ch_CustID,
      input tt_Trx.dt_TrxTimeStamp,
      input tt_Trx.in_Quantity,
      input tt_Trx.ch_CustName,
      input tt_Trx.ch_ItemName,
      input tt_Trx.de_TrxAmount,
      input tt_Trx.da_TrxDate,
      input tt_Trx.dt_PostTime
      ).
  find tt_Cust
      where tt_Cust.ch_CustName EQ tt_Trx.ch_CustName
      .
  lob_MyList:putValue(
       tt_Cust.ob_Cust,
       lob_CurrentTrx
       ).
  if lob_FirstTrx eq ?
  then lob_FirstTrx = lob_CurrentTrx.
end.
lob_LastTrx = lob_CurrentTrx.

message "Number of objects in ListMap = " + string(lob_MyList:size()) view-as alert-box.
message "Number of unique keys = " + string(lob_myList:NumberOfKeys()) view-as alert-box.
message "Is List empty? " + string(lob_MyList:isEmpty()) view-as alert-box.

message "List contains last object added = " + string(lob_MyList:containsValue( lob_LastTrx )) view-as alert-box.

message "The Last Key's Cust ID = " + string(cast(lob_MyList:getLastKey(), "com.cintegrity.Lib.Collection.CustID"):getCustID()) view-as alert-box.
message "The Last Cust ID = " + string(cast(lob_MyList:getLastValue(), "com.cintegrity.Lib.Collection.Trx"):getCustName()) view-as alert-box.

message "Positioned at end, then 'has next' = " + string(lob_MyList:hasNextKey()) + " and 'has prev' = " + string(lob_MyList:hasPrevKey() ) view-as alert-box.

lob_CurrentKey = cast(lob_MyList:getLastKey(), "com.cintegrity.Lib.Collection.CustID").
repeat:
  lob_CurrentTrx = cast(lob_myList:getNextValueSameKey(), "com.cintegrity.Lib.Collection.Trx").
  if not valid-object( lob_CurrentTrx )
  then leave.
end.
lin_CurrentSeq = lob_myList:getCurrentSeq().
lob_myList:removeValue( lob_CurrentKey, lin_CurrentSeq ).

message "After removing last Object, number of values = " + string(lob_MyList:size()) view-as alert-box.
message "The Last Key's Cust ID now is = " + string(cast(lob_MyList:getLastKey(), "com.cintegrity.Lib.Collection.CustID"):getCustID()) view-as alert-box.

message "The Next to the last Key's Cust ID = " + string(cast(lob_MyList:getPrevKey(), "com.cintegrity.Lib.Collection.CustID"):getCustID()) view-as alert-box.
message "The Next to the last Cust ID = " + string(cast(lob_MyList:getPrevValue(), "com.cintegrity.Lib.Collection.Trx"):getCustName()) view-as alert-box.

lob_CurrentTrx = cast(lob_MyList:getFirstValue(), "com.cintegrity.Lib.Collection.Trx").
message "Positioned at the beginning, then 'has next' = " + string(lob_MyList:hasNextKey() )  " and 'has prev' = " + string(lob_MyList:hasPrevKey()) view-as alert-box.

message "The First Key's Cust ID = " + string(cast(lob_MyList:getFirstKey(), "com.cintegrity.Lib.Collection.CustID"):getCustID()) view-as alert-box.
message "The First Cust ID = " + string(cast(lob_MyList:getFirstValue(), "com.cintegrity.Lib.Collection.Trx"):getCustName()) view-as alert-box.

message "The Next to the first Key's Cust ID = " + string(cast(lob_MyList:getNextKey(), "com.cintegrity.Lib.Collection.CustID"):getCustID()) view-as alert-box.
message "The Next to the first Cust ID = " + string(cast(lob_MyList:getNextValue(), "com.cintegrity.Lib.Collection.Trx"):getCustName()) view-as alert-box.

lin_TargetCount = random( 2, lob_myList:size() ) - 1.
lob_OldTrx = lob_myList:getFirstValue().
lob_CurrentTrx = cast(lob_OldTrx, "com.cintegrity.Lib.Collection.Trx").

do lin_Count = 1 to lin_TargetCount - 1:
  lob_CurrentTrx = cast(lob_MyList:getNextValue(), "com.cintegrity.Lib.Collection.Trx").
  if not valid-object( lob_CurrentTrx )
  then do:
    message "Unexpected error searching for random record" view-as alert-box.
    return.
  end.
end.
lob_CurrentKey = cast(lob_MyList:getCurrentKey(), "com.cintegrity.Lib.Collection.CustID").

message "The " + string(lin_TargetCount) + "th Key's Cust ID = " + string(lob_CurrentKey:getCustID()) view-as alert-box.
message "The " + string(lin_TargetCount) + "th Cust ID = " + string(cast(lob_MyList:getValue( lob_CurrentKey ), "com.cintegrity.Lib.Collection.Trx"):getCustName()) view-as alert-box.
message "List contains this Key = " + string(lob_MyList:containsKey( lob_CurrentKey )) view-as alert-box.
message "The current Key's Cust ID is " + string(cast(lob_MyList:getCurrentKey(), "com.cintegrity.Lib.Collection.CustID"):getCustID()) view-as alert-box.

lob_MyList:clearAll().
message "After clearing all Objects, number of values = " + string(lob_MyList:size()) view-as alert-box.

if valid-object(  lob_MyList )
then delete object lob_MyList.
if valid-object( lob_FirstTrx )
then delete object lob_FirstTrx.
if valid-object( lob_LastTrx )
then delete object lob_LastTrx.
if valid-object( lob_CurrentTrx )
then delete object lob_CurrentTrx.
if valid-object( lob_OldTrx )
then delete object lob_OldTrx.
assign
    lob_MyList = ?
    lob_FirstTrx = ?
    lob_LastTrx = ?
    lob_CurrentTrx = ?
    lob_OldTrx = ?
    .
