/* com.cintegrity.Lib.Collection.TestStringObjectSetMap.p */

define temp-table tt_Trx no-undo
                                                            /* unique values */
    field in_Seq as integer
    field ch_TrxID as character
    /* omitted unique decimal */
    /* omitted unique date */
    field dt_TrxTimeStamp as datetime
                                                        /* non-unique values */
    field in_Quantity as integer
    field ch_CustName as character
    field ch_ItemName as character
    field de_TrxAmount as decimal
    field da_TrxDate as date
    field dt_PostTime as datetime
    index ix_BySeq as primary unique in_Seq
    index ix_ByTimeStamp as unique dt_TrxTimeStamp
    .
define variable lhn_Trx as handle no-undo.
define variable lob_MySet as com.cintegrity.Lib.Collection.StringObjectSetMap no-undo.
define variable lch_TrxID as character no-undo.
define variable lob_CurrentTrx as com.cintegrity.Lib.Collection.Trx no-undo.
define variable lob_OldTrx as Progress.Lang.Object.
define variable lob_FirstTrx as com.cintegrity.Lib.Collection.Trx no-undo.
define variable lob_LastTrx as com.cintegrity.Lib.Collection.Trx no-undo.
define variable lch_InputFileName as character no-undo initial "com/cintegrity/Lib/Collection/TrxData.xml".
define variable lch_SchemaFileName as character no-undo initial "com/cintegrity/Lib/Collection/TrxDataSchema.xml".
define variable lch_CurrentKey as character no-undo.
define variable lin_TargetCount as integer no-undo.
define variable lin_Count as integer no-undo.
define variable llg_IsOK as logical no-undo.

if search( lch_InputFileName ) ne ?
then lch_InputFileName = search( lch_InputFileName ).
else do:
  message "Could not find test input file " + lch_InputFileName + " - aborting" view-as alert-box.
  return.
end.

if search( lch_SchemaFileName ) ne ?
then lch_SchemaFileName = search( lch_SchemaFileName ).
else do:
  message "Could not find Schema file " + lch_SchemaFileName + " - aborting" view-as alert-box.
  return.
end.

lhn_Trx = temp-table tt_Trx:handle.
lhn_Trx:read-xml(
    "FILE",
    lch_InputFileName,
    "empty",
    lch_SchemaFileName,
    false
    ).

lob_MySet = new com.cintegrity.Lib.Collection.StringObjectSetMap().

for each tt_Trx:
  lob_CurrentTrx = new com.cintegrity.Lib.Collection.Trx(
      input tt_Trx.in_Seq,
      input tt_Trx.ch_TrxID,
      input tt_Trx.dt_TrxTimeStamp,
      input tt_Trx.in_Quantity,
      input tt_Trx.ch_CustName,
      input tt_Trx.ch_ItemName,
      input tt_Trx.de_TrxAmount,
      input tt_Trx.da_TrxDate,
      input tt_Trx.dt_PostTime
      ).
  lob_OldTrx = lob_MySet:putValue(
       tt_Trx.ch_TrxID,
       lob_CurrentTrx
       ).
  if not lob_OldTrx EQ ?
  then do:
    message "Unexpected error creating initial set map" view-as alert-box.
    return.
  end.
  if lob_FirstTrx eq ?
  then lob_FirstTrx = lob_CurrentTrx.
end.
lob_LastTrx = lob_CurrentTrx.

message "Number of objects in SetMap = " + string(lob_MySet:size()) view-as alert-box.

message "Is set empty? " + string(lob_MySet:isEmpty()) view-as alert-box.

message "Set contains last object added = " + string(lob_MySet:containsValue( lob_LastTrx )) view-as alert-box.

lch_TrxID = lob_MySet:getLastKey().
message "The Last Key = " + lob_MySet:getLastKey() view-as alert-box.
message "The Last TrxID = " + cast(lob_MySet:getLastValue(), "com.cintegrity.Lib.Collection.Trx"):getTrxID() view-as alert-box.

message "Positioned at end, then 'has next' = " + string(lob_MySet:hasNextKey()) + " and 'has prev' = " + string(lob_MySet:hasPrevKey() ) view-as alert-box.

lob_OldTrx = lob_MySet:removeValue( lch_TrxID ).
if lob_OldTrx eq ?
then do:
  message "Unexpected error on remove of last value" view-as alert-box.
  return.
end.

message "After removing last Object, number of values = " + string(lob_MySet:size()) view-as alert-box.
message "The Last Key now is = " + lob_MySet:getLastKey() view-as alert-box.

message "The Next to the last Key = " + lob_MySet:getPrevKey() view-as alert-box.
message "The Next to the last TrxID = " + cast(lob_MySet:getPrevValue(), "com.cintegrity.Lib.Collection.Trx"):getTrxID() view-as alert-box.

lob_CurrentTrx = cast(lob_MySet:getFirstValue(), "com.cintegrity.Lib.Collection.Trx").
message "Positioned at the beginning, then 'has next' = " + string(lob_MySet:hasNextKey() )  " and 'has prev' = " + string(lob_MySet:hasPrevKey() ) view-as alert-box.

message "The First Key = " + lob_MySet:getFirstKey() view-as alert-box.
message "The First TrxID = " + cast(lob_MySet:getFirstValue(), "com.cintegrity.Lib.Collection.Trx"):getTrxID() view-as alert-box.

message "The Next to the first Key = " + lob_MySet:getNextKey() view-as alert-box.
message "The Next to the first TrxID = " + cast(lob_MySet:getNextValue(), "com.cintegrity.Lib.Collection.Trx"):getTrxID() view-as alert-box.


lin_TargetCount = random( 1, lob_MySet:size() ) - 1.
lob_OldTrx = lob_MySet:getFirstValue().

do lin_Count = 1 to lin_TargetCount - 1:
  lch_CurrentKey = cast(lob_MySet:getNextValue(), "com.cintegrity.Lib.Collection.Trx"):getTrxID().
  if lch_CurrentKey eq ?
  then do:
    message "Unexpected error searching for random record" view-as alert-box.
    return.
  end.
end.

message "The " + string(lin_TargetCount) + "th Key = " + lch_CurrentKey view-as alert-box.
message "The " + string(lin_TargetCount) + "th TrxID = " + cast(lob_MySet:getValue( lch_CurrentKey ), "com.cintegrity.Lib.Collection.Trx"):getTrxID() view-as alert-box.
message "Set contains this key = " + string(lob_MySet:containsKey( lch_CurrentKey )) view-as alert-box.
message "The current Key is " + lob_MySet:getCurrentKey() view-as alert-box.

lob_MySet:clearAll().
message "After clearing all Objects, number of values = " + string(lob_MySet:size()) view-as alert-box.
if valid-object(  lob_MySet )
then delete object lob_MySet.
if valid-object( lob_FirstTrx )
then delete object lob_FirstTrx.
if valid-object( lob_LastTrx )
then delete object lob_LastTrx.
if valid-object( lob_CurrentTrx )
then delete object lob_CurrentTrx.
if valid-object( lob_OldTrx )
then delete object lob_OldTrx.
assign
    lob_MySet = ?
    lob_FirstTrx = ?
    lob_LastTrx = ?
    lob_CurrentTrx = ?
    lob_OldTrx = ?
    .
